# This file is part of FatNS, a DNS sniffing and attack detection tool.
# Copyright (C) 2006 Ohad Lutzky and Boaz Goldstein
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

module FatNS
  module Truisms
    QCLASS_TABLE = [
    ["ERROR","Reserved"  ,  "[IANA]"], # 0
    ["IN","Internet (IN)"  ,  "[RFC1035]"], # 1
    ["ERROR","Unassigned"  ,  "[IANA]"], # 2
    ["CH","Chaos (CH)"  ,  "[RFC1035]"], # 3
    ["HS","Hesiod (HS)"  ,  "[RFC1035]"], # 4

    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],
    ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''], ['','',''],

    ["ERROR","None"  ,  "[RFC2136]"], # 254
    ["ANY","Any [QCLASS Only]"  ,  "[RFC1035]"] # 255
    ]

    # This function gives you the matching DNS query class id
    # for a given type identifier string, +s+.
    def Truisms.get_qclass_id(s)
      QCLASS_TABLE.each_with_index do |t,i|
        return i if (not t.nil?) and t[0] == s
      end
      raise "Invalid DNS query class #{_type}"
    end
  end
end
